
#include <nds.h>

#include "../../ipc6.h"

#undef SOUND_FREQ
#define SOUND_FREQ(n)	(0x10000 - (16756000 / (n)))

#include "main_a7sleep.h"

void InterruptHandler_VBlank(void)
{
  IPC6->heartbeat++;
  
  uint16 but=0, xpx=0, ypx=0, batt=0, aux=0;
  int t1=0, t2=0;
  uint32 temp=0;
  uint8 ct[sizeof(IPC6->curtime)];
  
  // Read the X/Y buttons and the /PENIRQ line
  but = REG_KEYXY;
  
  touchPosition touchPos=touchReadXY();
  xpx=touchPos.px;
  ypx=touchPos.py;
  
//  batt = touchRead(TSC_MEASURE_BATTERY);
//  aux  = touchRead(TSC_MEASURE_AUX);
  
  // Read the time
  rtcGetTime((uint8 *)ct);
  BCDToInteger((uint8 *)&(ct[1]), 7);
  
  for(u32 i=0; i<sizeof(ct); i++) {
    IPC6->curtime[i] = ct[i];
  }
  
  
  // Read the temperature
//  temp = touchReadTemperature(&t1, &t2);
  
  // Update the IPC struct
  IPC6->buttons   = but;
  IPC6->touchXpx  = xpx;
  IPC6->touchYpx  = ypx;
//  IPC6->battery   = batt;
//  IPC6->aux       = aux;
  
//  IPC6->temperature = temp;
//  IPC6->tdiode1 = t1;
//  IPC6->tdiode2 = t2;
}

// -------------------------------------------------

// read firmware.
// reference hbfirmware.zip/firmware/arm7/source/settings.c

//typedef void(*call0)(void);
typedef void(*call3)(u32,void*,u32);

//size must be a multiple of 4
static inline void read_nvram(u32 src, void *dst, u32 size) {
	((call3)0x2437)(src,dst,size);
}

//read firmware settings
static inline void load_PersonalData() {
	u32 src, count0, count1;

	read_nvram(0x20, &src, 4);		//find settings area
	src=(src&0xffff)*8;

	read_nvram(src+0x70, &count0, 4);	//pick recent copy
	read_nvram(src+0x170, &count1, 4);
	if((u16)count0<(u16)count1){
		src+=0x100;
	}
	
	read_nvram(src, PersonalData, 0x80);
	if(swiCRC16(0xffff,PersonalData,0x70) != ((u16*)PersonalData)[0x72/2]){ 	//invalid / corrupt?
		read_nvram(src^0x100, PersonalData, 0x80);	//try the older copy
	}
}

static inline void main_InitIRQ(void)
{
  REG_IME = 0;
  irqInit();
  irqSet(IRQ_VBLANK,(VoidFunctionPointer)InterruptHandler_VBlank);
  DISP_SR |= DISP_VBLANK_IRQ ;
  REG_IE = IRQ_VBLANK;
  REG_IME = 1;
}

static inline void main_InitSoundDevice(void)
{
  powerON(POWER_SOUND);
  SOUND_CR = SOUND_ENABLE | SOUND_VOL(0x7F);
  
  POWER_CR&=~POWER_UNKNOWN; // wifi power off
  
  swiChangeSoundBias(1,0x400);
  a7SetSoundAmplifier(true);
}

__attribute__((noinline)) static void main_InitAll(void)
{
  REG_IME=0;
  REG_IE=0;
  
  // Clear DMA
  for(int i=0;i<0x30/4;i++){
    *((vu32*)(0x40000B0+i))=0;
  }
  
  {
    u32 *pbuf=(u32*)IPC6;
    u32 size=sizeof(TransferRegion6);
    for(u32 idx=1;idx<size/4;idx++){ // idx==1 is heartbeat
      pbuf[idx]=0;
    }
  }
  
  IPC6->heartbeat=0;
  IPC6->UserLanguage=(u32)-1;
  
  // Reset the clock if needed
  rtcReset();
  
  REG_SPICNT = SPI_ENABLE|SPI_CONTINUOUS|SPI_DEVICE_NVRAM;
  load_PersonalData();
  REG_SPICNT = 0;
  IPC6->UserLanguage=PersonalData->language;
  
  a7lcd_select(PM_BACKLIGHT_BOTTOM | PM_BACKLIGHT_TOP);
  a7led(0);
  
  IPC6->IPCSND=EIPCSND_Null;
  
  main_InitIRQ();
  
  main_InitSoundDevice();
}

//---------------------------------------------------------------------------------
int main(void) {
//---------------------------------------------------------------------------------
  
  IPC6->heartbeat=0;
  main_InitAll();
  while(IPC6->heartbeat==0);
  
  u32 sch=0;
  u32 sloopch=14;
  
  // Keep the ARM7 out of main RAM
  while(1){
    swiWaitForVBlank();
    
    if(IPC6->IPCSND!=EIPCSND_Null){
      switch(IPC6->IPCSND){
        case EIPCSND_Null: break;
        case EIPCSND_AllStop: {
          for(u32 ch=0;ch<16;ch++){
            SCHANNEL_CR(ch)=0;
          }
        } break;
        case EIPCSND_Play: {
          sch+=2;
          if(sch==sloopch) sch=0;
          
          SCHANNEL_TIMER(sch+0)  = SOUND_FREQ(IPC6->IPCSND_Freq);
          SCHANNEL_SOURCE(sch+0) = (u32)IPC6->IPCSND_lbuf;
          SCHANNEL_LENGTH(sch+0) = IPC6->IPCSND_BufCount >> 1 ; // for 16bit
          SCHANNEL_TIMER(sch+1)  = SOUND_FREQ(IPC6->IPCSND_Freq);
          SCHANNEL_SOURCE(sch+1) = (u32)IPC6->IPCSND_rbuf;
          SCHANNEL_LENGTH(sch+1) = IPC6->IPCSND_BufCount >> 1 ; // for 16bit
          SCHANNEL_CR(sch+0)     = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(127) | SOUND_PAN(0) | SOUND_16BIT;
          SCHANNEL_CR(sch+1)     = SCHANNEL_ENABLE | SOUND_ONE_SHOT | SOUND_VOL(127) | SOUND_PAN(127) | SOUND_16BIT;
        } break;
        case EIPCSND_PlayLoop: {
          SCHANNEL_TIMER(sloopch+0)  = SOUND_FREQ(IPC6->IPCSND_Freq);
          SCHANNEL_SOURCE(sloopch+0) = (u32)IPC6->IPCSND_lbuf;
          SCHANNEL_LENGTH(sloopch+0) = IPC6->IPCSND_BufCount >> 1 ; // for 16bit
          SCHANNEL_TIMER(sloopch+1)  = SOUND_FREQ(IPC6->IPCSND_Freq);
          SCHANNEL_SOURCE(sloopch+1) = (u32)IPC6->IPCSND_rbuf;
          SCHANNEL_LENGTH(sloopch+1) = IPC6->IPCSND_BufCount >> 1 ; // for 16bit
          SCHANNEL_CR(sloopch+0)     = SCHANNEL_ENABLE | SOUND_REPEAT | SOUND_VOL(127) | SOUND_PAN(0) | SOUND_16BIT;
          SCHANNEL_CR(sloopch+1)     = SCHANNEL_ENABLE | SOUND_REPEAT | SOUND_VOL(127) | SOUND_PAN(127) | SOUND_16BIT;
        } break;
      }
      IPC6->IPCSND=EIPCSND_Null;
    }
  }
  
  return(0);
}

