
#include <stdio.h>

#include <nds.h>
#include "gba_nds_fat.h"

#include "_console.h"
#include "_const.h"

#include "memtool.h"

#include "glib/glib.h"

#include "bitmap.h"

#define BI_RGB (0)
#define BI_RLE8 (1)
#define BI_RLE4 (2)
#define BI_Bitfields (3)

typedef struct {
  u8 bfType[2];
  u32 bfSize;
  u16 bfReserved1;
  u16 bfReserved2;
  u32 bfOffset;
  u32 biSize;
  u32 biWidth;
  u32 biHeight;
  u16 biPlanes;
  u16 biBitCount;
  u32 biCopmression;
  u32 biSizeImage;
  u32 biXPixPerMeter;
  u32 biYPixPerMeter;
  u32 biClrUsed;
  u32 biCirImportant;
} TBMPHeader;

TBitmap* Bitmap_LoadFromFile(char *fn,bool TransFlag)
{
  TBitmap *pbm=(TBitmap*)safemalloc(sizeof(TBitmap));
  if(pbm==NULL) return(NULL);
  MemSet8DMA3(0,pbm,sizeof(TBitmap));
  
  pbm->TransFlag=TransFlag;
  
  FAT_FILE *fp=FAT_fopen(fn,"r");
  
  if(fp==NULL){
    safefree(pbm); pbm=NULL;
    return(pbm);
  }
  
  TBMPHeader head;
  
  FAT_fseek(fp,0,SEEK_SET);
  
#define get8() rbuf[rofs+0]; rofs++;
#define get16() ((u16)rbuf[rofs+0])|(((u16)rbuf[rofs+1])<<8); rofs+=2;
#define get32() ((u16)rbuf[rofs+0])|(((u16)rbuf[rofs+1])<<8)|(((u16)rbuf[rofs+2])<<16)|(((u16)rbuf[rofs+3])<<24); rofs+=4;
  
  {
    u8 rbuf[54];
    u32 rofs=0;
    
    FAT_fread(rbuf,1,54,fp);
    
    head.bfType[0]=get8();
    head.bfType[1]=get8();
    head.bfSize=get32();
    head.bfReserved1=get16();
    head.bfReserved2=get16();
    head.bfOffset=get32();
    head.biSize=get32();
    head.biWidth=get32();
    head.biHeight=get32();
    head.biPlanes=get16();
    head.biBitCount=get16();
    head.biCopmression=get32();
    head.biSizeImage=get32();
    head.biXPixPerMeter=get32();
    head.biYPixPerMeter=get32();
    head.biClrUsed=get32();
    head.biCirImportant=get32();
  }
  
#undef get8
#undef get16
#undef get32
  
  u16 Palette[256];
  
  switch(head.biBitCount){
    case 4: {
      u32 rbuf[16];
      FAT_fread(rbuf,4,16,fp);
      for(int idx=0;idx<16;idx++){
        u32 data=rbuf[idx];
        if(data==0){
          Palette[idx]=RGB15(0,0,0)|BIT15;
          }else{
          Palette[idx]=RGB15(((data>>16)&0xff)>>3,((data>>8)&0xff)>>3,((data>>0)&0xff)>>3)|BIT15;
        }
      }
    } break;
    case 8: {
      u32 rbuf[256];
      FAT_fread(rbuf,4,256,fp);
      for(int idx=0;idx<256;idx++){
        u32 data=rbuf[idx];
        if(data==0){
          Palette[idx]=RGB15(0,0,0)|BIT15;
          }else{
          Palette[idx]=RGB15(((data>>16)&0xff)>>3,((data>>8)&0xff)>>3,((data>>0)&0xff)>>3)|BIT15;
        }
      }
    } break;
    case 24: break;
    default: break;
  }
  
  if((head.bfType[0]!='B')||(head.bfType[1]!='M')){
    _consolePrintf("Error MagicID!=BM");
    safefree(pbm); pbm=NULL;
    return(pbm);
  }
  
  if(head.biCopmression!=BI_RGB){
    _consolePrintf("Error notsupport Compression");
    safefree(pbm); pbm=NULL;
    return(pbm);
  }
  
  if(head.biHeight>=0x80000000){
    _consolePrintf("Error notsupport OS/2 format");
    safefree(pbm); pbm=NULL;
    return(pbm);
  }
  
  if(head.biPlanes!=1){
    _consolePrintf("Error notsupport Planes!=1");
    safefree(pbm); pbm=NULL;
    return(pbm);
  }
  
  u32 DataWidth=0;
  
  switch(head.biBitCount){
    case 4: DataWidth=(head.biWidth+1)/2; break;
    case 8: DataWidth=head.biWidth*1; break;
    case 24: DataWidth=head.biWidth*3; break;
    default: {
      _consolePrintf("Error notsupport BitCount %d!=8/24",head.biBitCount);
      safefree(pbm); pbm=NULL;
      return(pbm);
    } break;
  }
  
  if((DataWidth&3)!=0){
    DataWidth+=4-(DataWidth&3);
  }
  
  u32 w=head.biWidth;
  u32 h=head.biHeight;
  
  pbm->Width=w;
  pbm->Height=h;
  
  pbm->pBM=(u16*)safemalloc(h*w*2);
  if(pbm->pBM==NULL){
    _consolePrintf("Fatal error. Memory overflow.\n");
    safefree(pbm); pbm=NULL;
    return(pbm);
  }
  
  if(TransFlag==true) MemSet16DMA3(0,pbm->pBM,h*w*2);
  
  u8 *psrcbuf=(u8*)safemalloc(h*DataWidth);
  if(psrcbuf==NULL){
    _consolePrintf("Fatal error. Memory overflow.\n");
    ShowLogHalt();
  }
  
  FAT_fseek(fp,head.bfOffset,SEEK_SET);
  FAT_fread(&psrcbuf[0],1,h*DataWidth,fp);
  
  switch(head.biBitCount){
    case 4: {
      if(TransFlag==true){
        u32 tpal=psrcbuf[0]&0x0f;
        
        for(u32 y=0;y<h;y++){
          u8 *psrcbm=&psrcbuf[y*DataWidth];
          u16 *pdstbm=&pbm->pBM[(h-1-y)*w];
          for(u32 x=w/2;x!=0;x--){
            u32 data=psrcbm[0];
            if(tpal!=((data>>4)&0x0f)) *pdstbm=Palette[(data>>4)&0x0f];
            pdstbm++;
            if(tpal!=((data>>0)&0x0f)) *pdstbm=Palette[(data>>0)&0x0f];
            pdstbm++;
            psrcbm++;
          }
          if((w&1)!=0){
            u32 data=psrcbm[0];
            if(tpal!=((data>>4)&0x0f)) *pdstbm=Palette[(data>>4)&0x0f];
          }
        }
        }else{
        for(u32 y=0;y<h;y++){
          u8 *psrcbm=&psrcbuf[y*DataWidth];
          u16 *pdstbm=&pbm->pBM[(h-1-y)*w];
          for(u32 x=w/2;x!=0;x--){
            u32 data=psrcbm[0];
            *pdstbm=Palette[(data>>4)&0x0f];
            pdstbm++;
            *pdstbm=Palette[(data>>0)&0x0f];
            pdstbm++;
            psrcbm++;
          }
          if((w&1)!=0){
            u32 data=psrcbm[0];
            *pdstbm=Palette[(data>>4)&0x0f];
          }
        }
      }
    } break;
    case 8: {
      if(TransFlag==true){
        u8 tpal=psrcbuf[0];
        
        for(u32 y=0;y<h;y++){
          u8 *psrcbm=&psrcbuf[y*DataWidth];
          u16 *pdstbm=&pbm->pBM[(h-1-y)*w];
          for(u32 x=w;x!=0;x--){
            if(tpal!=psrcbm[0]) *pdstbm=Palette[psrcbm[0]];
            psrcbm++;
            pdstbm++;
          }
        }
        }else{
        for(u32 y=0;y<h;y++){
          u8 *psrcbm=&psrcbuf[y*DataWidth];
          u16 *pdstbm=&pbm->pBM[(h-1-y)*w];
          for(u32 x=w;x!=0;x--){
            *pdstbm=Palette[psrcbm[0]];
            psrcbm++;
            pdstbm++;
          }
        }
      }
    } break;
    case 24: {
      if(TransFlag==true){
        u8 tb=psrcbuf[0],tg=psrcbuf[1],tr=psrcbuf[2];
        
        for(u32 y=0;y<h;y++){
          u8 *psrcbm=&psrcbuf[y*DataWidth];
          u16 *pdstbm=&pbm->pBM[(h-1-y)*w];
          for(u32 x=w;x!=0;x--){
            if((tb!=psrcbm[0])||(tg!=psrcbm[1])||(tr!=psrcbm[2])) *pdstbm=RGB15(psrcbm[2]>>3,psrcbm[1]>>3,psrcbm[0]>>3)|BIT15;
            psrcbm+=3;
            pdstbm++;
          }
        }
        }else{
        for(u32 y=0;y<h;y++){
          u8 *psrcbm=&psrcbuf[y*DataWidth];
          u16 *pdstbm=&pbm->pBM[(h-1-y)*w];
          for(u32 x=w;x!=0;x--){
            *pdstbm=RGB15(psrcbm[2]>>3,psrcbm[1]>>3,psrcbm[0]>>3)|BIT15;
            psrcbm+=3;
            pdstbm++;
          }
        }
      }
    } break;
    default: break;
  }
  
  safefree(psrcbuf); psrcbuf=NULL;
  
  FAT_fclose(fp);
  
  return(pbm);
}

void Bitmap_Free(TBitmap *pbm)
{
  if(pbm==NULL) return;
  
  if(pbm->pBM!=NULL){
    safefree(pbm->pBM); pbm->pBM=NULL;
  }
  
  safefree(pbm); // pbm=NULL;
}

void Bitmap_Draw(TBitmap *pbm,CglCanvas *pCanvas,u32 dx,u32 dy)
{
  u16 *pSrcBM=pbm->pBM;
  u32 w=pbm->Width;
  u32 h=pbm->Height;
  
  u16 *pDstBM=pCanvas->GetVRAMBuf();
  u32 dstw=pCanvas->GetWidth();
  
  pDstBM+=(dy*dstw)+dx;
  
  if(pbm->TransFlag==true){
    for(u32 y=0;y<h;y++){
      for(u32 x=0;x<w;x++){
        if(*pSrcBM!=0) pDstBM[x]=*pSrcBM;
        pSrcBM++;
      }
      pDstBM+=dstw;
    }
    }else{
    for(u32 y=0;y<h;y++){
      for(u32 x=0;x<w;x++){
        pDstBM[x]=*pSrcBM;
        pSrcBM++;
      }
      pDstBM+=dstw;
    }
  }
}

void Bitmap_Draw2(TBitmap *pbm,u32 sx,u32 sy,CglCanvas *pCanvas,u32 dx,u32 dy,u32 dw,u32 dh)
{
  u16 *pSrcBM=pbm->pBM;
  u32 w=pbm->Width;
  
  pSrcBM+=(sy*w)+sx;
  
  u16 *pDstBM=pCanvas->GetVRAMBuf();
  u32 dstw=pCanvas->GetWidth();
  
  pDstBM+=(dy*dstw)+dx;
  
  if(pbm->TransFlag==true){
    for(u32 y=0;y<dh;y++){
      for(u32 x=0;x<dw;x++){
        if(pSrcBM[x]!=0) pDstBM[x]=pSrcBM[x];
      }
      pSrcBM+=w;
      pDstBM+=dstw;
    }
    }else{
    for(u32 y=0;y<dh;y++){
      for(u32 x=0;x<dw;x++){
        pDstBM[x]=pSrcBM[x];
      }
      pSrcBM+=w;
      pDstBM+=dstw;
    }
  }
}

