
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <NDS.h>

#include "_console.h"
#include "_const.h"

#include "inifile.h"
#include "memtool.h"

TGlobalINI GlobalINI;

void InitINI(void)
{
  {
    TiniSystem *System=&GlobalINI.System;
    System->EnableFadeWindow=true;
    System->EnableSE=true;
    System->SplashScreen=true;
  }
  
  {
    TiniKeyRepeat *KeyRepeat=&GlobalINI.KeyRepeat;
    KeyRepeat->DelayCount=15;
    KeyRepeat->RateCount=1;
  }
  
  {
    TiniNewFilename *NewFilename=&GlobalINI.NewFilename;
    NewFilename->AddGameID=true;
    NewFilename->AddENGName=true;
    NewFilename->AddDateYear=false;
    NewFilename->AddDateMonth=true;
    NewFilename->AddDateDay=true;
    NewFilename->AddTimeHour=true;
    NewFilename->AddTimeMin=true;
    NewFilename->AddTimeSec=false;
    NewFilename->ENGNameLimit=12;
  }

}

static char section[128];
static u32 readline;

static void readsection(char *str)
{
  str++;
  
  u32 ofs;
  
  ofs=0;
  while(*str!=']'){
    if((128<=ofs)||(*str==0)){
      _consolePrintf("line%d error.\nThe section name doesn't end correctly.\n",readline);
      ShowLogHalt();
    }
    section[ofs]=*str;
    str++;
    ofs++;
  }
  section[ofs]=0;
}

static void readkey(char *str)
{
  if(section[0]==0){
    _consolePrintf("line%d error.\nThere is a key ahead of the section name.\n",readline);
//    ShowLogHalt();
    return;
  }
  
  char key[128],value[128];
  
  u32 ofs;
  
  ofs=0;
  while(*str!='='){
    if((128<=ofs)||(*str==0)){
      _consolePrintf("line%d error.\nThe key name doesn't end correctly.\n",readline);
      ShowLogHalt();
    }
    key[ofs]=*str;
    str++;
    ofs++;
  }
  key[ofs]=0;
  
  str++;
  
  ofs=0;
  while(*str!=0){
    if(128<=ofs){
      _consolePrintf("line%d error.\nThe value doesn't end correctly.\n",readline);
      ShowLogHalt();
    }
    value[ofs]=*str;
    str++;
    ofs++;
  }
  value[ofs]=0;
  
  s32 ivalue=atoi(value);
  bool bvalue;
  
  if(ivalue==0){
    bvalue=false;
    }else{
    bvalue=true;
  }
  
  if(strcmp(section,"System")==0){
    TiniSystem *System=&GlobalINI.System;
    
    if(strcmp(key,"EnableFadeWindow")==0){
      System->EnableFadeWindow=bvalue;
      return;
    }
    if(strcmp(key,"EnableSE")==0){
      System->EnableSE=bvalue;
      return;
    }
    if(strcmp(key,"SplashScreen")==0){
      System->SplashScreen=bvalue;
      return;
    }
    
  }
  
  if(strcmp(section,"KeyRepeat")==0){
    TiniKeyRepeat *KeyRepeat=&GlobalINI.KeyRepeat;
    
    if(strcmp(key,"DelayCount")==0){
      if(ivalue!=0) KeyRepeat->DelayCount=ivalue;
      return;
    }
    if(strcmp(key,"RateCount")==0){
      if(ivalue!=0) KeyRepeat->RateCount=ivalue;
      return;
    }
    
  }
  
  if(strcmp(section,"NewFilename")==0){
    TiniNewFilename *NewFilename=&GlobalINI.NewFilename;
    
    if(strcmp(key,"AddGameID")==0){
      NewFilename->AddGameID=bvalue;
      return;
    }
    if(strcmp(key,"AddENGName")==0){
      NewFilename->AddENGName=bvalue;
      return;
    }
    if(strcmp(key,"AddDateYear")==0){
      NewFilename->AddDateYear=bvalue;
      return;
    }
    if(strcmp(key,"AddDateMonth")==0){
      NewFilename->AddDateMonth=bvalue;
      return;
    }
    if(strcmp(key,"AddDateDay")==0){
      NewFilename->AddDateDay=bvalue;
      return;
    }
    if(strcmp(key,"AddTimeHour")==0){
      NewFilename->AddTimeHour=bvalue;
      return;
    }
    if(strcmp(key,"AddTimeMin")==0){
      NewFilename->AddTimeMin=bvalue;
      return;
    }
    if(strcmp(key,"AddTimeSec")==0){
      NewFilename->AddTimeSec=bvalue;
      return;
    }
    if(strcmp(key,"ENGNameLimit")==0){
      NewFilename->ENGNameLimit=ivalue;
      return;
    }
    
  }
  
  _consolePrintf("line%d error.\ncurrent section [%s] unknown key=%s value=%s\n",readline,section,key,value);
//  ShowLogHalt();
}

static void internal_LoadGlobalINI(char *pini,u32 inisize)
{
  section[0]=0;
  readline=0;
  
  u32 iniofs=0;
  
  while(iniofs<inisize){
    
    readline++;
    
    u32 linelen=0;
    
    // Calc Line Length
    {
      char *s=&pini[iniofs];
      
      while(0x20<=*s){
        linelen++;
        s++;
        if(inisize<=(iniofs+linelen)) break;
      }
      *s=0;
    }
    
    if(linelen!=0){
      char c=pini[iniofs];
      if((c==';')||(c=='/')||(c=='!')){
        // comment line
        }else{
        if(c=='['){
          readsection(&pini[iniofs]);
          }else{
          readkey(&pini[iniofs]);
        }
      }
    }
    
    iniofs+=linelen;
    
    // skip NULL,CR,LF
    {
      char *s=&pini[iniofs];
      
      while(*s<0x20){
        iniofs++;
        s++;
        if(inisize<=iniofs) break;
      }
    }
    
  }
}

void LoadINI(char *pbuf,u32 size)
{
  if((pbuf==NULL)||(size==0)) return;
  
  internal_LoadGlobalINI(pbuf,size);
}

