
#include <NDS.h>
#include <NDS/ARM9/CP15.h>
#include "../../ipc6.h"

#include "_console.h"
#include "_consolePrf.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "memtool.h"

#include "maindef.h"
#include "inifile.h"

#include "glib/glib.h"

#include "arm9tcm.h"

#include "gba_nds_fat.h"
#include "mediatype.h"

#include "_const.h"
#include "sound.h"
#include "window.h"
#include "lang.h"

#include "cardme.h"
#include "savename.h"

#include "setarm9_reg_waitcr.h"

#include "bitmap.h"

extern void videoSetModeSub_SetShowLog(bool e);

// ------------------------------------------------------

static TGameData GameDataDummy={0,"----" ,"unknown",NULL,0x0000,NULL,0};

enum EFileType {EFT_None=0,EFT_Back=1,EFT_Path=2,EFT_File=3};

typedef struct {
  char lfn[MAX_FILENAME_LENGTH];
  EFileType EFT;
  u32 FileSize;
  TGameData *pGameData;
  u16 *IconBM;
} TFile;

static int FilesCount;
static TFile *pFiles;

void WaitKeyClear(void);
void WaitKeyDown(void);
void WaitKeyPress(void);

#include "main_exec_head.h"
#include "main_icon.h"
#include "main_dscard.h"
#include "main_dialog.h"
#include "main_filelist.h"

// ------------------------------------------------------------------

// ------------------------------------------------------------------

void ShowLogHalt(void)
{
  videoSetModeSub(MODE_2_2D | DISPLAY_BG2_ACTIVE);
  _consolePrintf("\n");
  while(1){
    swiWaitForVBlank();
  }
}

// ------------------------------------------------------

static u32 VsyncPassedCount;

void InterruptHandler_VBlank(void)
{
  VsyncPassedCount++;
}

void InitInterrupts(void)
{
  REG_IME = 0;
  VsyncPassedCount=0;
  irqInit();
  irqSet(IRQ_VBLANK,(VoidFunctionPointer)InterruptHandler_VBlank);
  REG_IME = 1;
}

// ------------------------------------------------------

void LoadSystemFont(void)
{
  CglFont *pglFont;
  CglUnicode *pglUnicode;
  
  {
    FAT_FILE *fp=FAT_fopen("/ReinMoon/font12.glf","r");
    if(fp==NULL){
      _consolePrintf("Fatal error!! font12.glf not found.\n");
      ShowLogHalt();
    }
    
    FAT_fseek(fp,0,SEEK_END);
    u32 size=FAT_ftell(fp);
    FAT_fseek(fp,0,SEEK_SET);
    
    u8 *buf=(u8*)safemalloc(size);
    FAT_fread(buf,1,size,fp);
    
    FAT_fclose(fp);
    
    pglFont=new CglFont(buf,size);
    
    safefree(buf); buf=NULL;
  }
  
  {
    FAT_FILE *fp=FAT_fopen("/ReinMoon/font12.glu","r");
    if(fp==NULL){
      _consolePrintf("Fatal error!! font12.glu not found.\n");
      ShowLogHalt();
    }
    
    FAT_fseek(fp,0,SEEK_END);
    u32 size=FAT_ftell(fp);
    FAT_fseek(fp,0,SEEK_SET);
    
    u8 *buf=(u8*)safemalloc(size);
    FAT_fread(buf,1,size,fp);
    
    FAT_fclose(fp);
    
    pglUnicode=new CglUnicode(buf,size);
    
    safefree(buf); buf=NULL;
  }
  
  pScreenMain->pCanvas->SetCglFont(pglFont);
  pScreenMain->pCanvas->SetCglUnicode(pglUnicode);
  pScreenMainOverlay->pCanvas->SetCglFont(pglFont);
  pScreenMainOverlay->pCanvas->SetCglUnicode(pglUnicode);
  pScreenSub->pCanvas->SetCglFont(pglFont);
  pScreenSub->pCanvas->SetCglUnicode(pglUnicode);
  
  StrConvert_SetUnicode(pglUnicode);
}

void LoadGlobalINI(void)
{
  FAT_FILE *fp=FAT_fopen("/ReinMoon/global.ini","r");
  if(fp==NULL){
    _consolePrintf("Fatal error!! global.ini not found.\n");
    ShowLogHalt();
  }
  
  FAT_fseek(fp,0,SEEK_END);
  u32 size=FAT_ftell(fp);
  FAT_fseek(fp,0,SEEK_SET);
  
  char *buf=(char*)safemalloc(size);
  FAT_fread(buf,1,size,fp);
  
  FAT_fclose(fp);
  
  LoadINI(buf,size);
  
  safefree(buf); buf=NULL;
}

void LoadSaveName(void)
{
  FAT_FILE *fp=FAT_fopen("/ReinMoon/savename.txt","r");
  if(fp==NULL){
    _consolePrintf("Fatal error!! savename.txt not found.\n");
    ShowLogHalt();
  }
  
  FAT_fseek(fp,0,SEEK_END);
  u32 size=FAT_ftell(fp);
  FAT_fseek(fp,0,SEEK_SET);
  
  char *buf=(char*)safemalloc(size);
  FAT_fread(buf,1,size,fp);
  
  FAT_fclose(fp);
  
  LoadGameDatas(buf,size);
  
  safefree(buf); buf=NULL;
  
  CreateGameDatasFind();
}

// ------------------------------------------------------

int main(void)
{
//  ResetMemory();
  
  REG_IME=0;
  
  POWER_CR = POWER_ALL_2D;
  
  glSetFuncDebugPrint(_consolePrint);
  glDefaultMemorySetting();
  
  {
    SUB_BG2_CR = BG_256_COLOR | BG_RS_64x64 | BG_MAP_BASE(8) | BG_TILE_BASE(0) | BG_PRIORITY_0; // Tile16kb Map2kb(64x32)
    
    BG_PALETTE_SUB[(0*16)+0] = RGB15(0,0,0); // unuse (transparent)
    BG_PALETTE_SUB[(0*16)+1] = RGB15(0,0,8) | BIT(15); // BG color
    BG_PALETTE_SUB[(0*16)+2] = RGB15(0,0,0) | BIT(15); // Shadow color
    BG_PALETTE_SUB[(0*16)+3] = RGB15(31,31,31) | BIT(15); // Text color
    
    u16 XDX=(u16)((8.0/6)*0x100);
    u16 YDY=(u16)((8.0/6)*0x100);
    
    SUB_BG2_XDX = XDX;
    SUB_BG2_XDY = 0;
    SUB_BG2_YDX = 0;
    SUB_BG2_YDY = YDY;
    
    SUB_BG2_CX=-1;
    SUB_BG2_CY=-1;
    
    //consoleInit() is a lot more flexible but this gets you up and running quick
    _consoleInitDefault((u16*)(SCREEN_BASE_BLOCK_SUB(8)), (u16*)(CHAR_BASE_BLOCK_SUB(0)));
  }
  
  glDefaultClassCreate();
  SetLang(EL_JPN);
  InitINI();
  
  _consolePrintf("boot %s %s\n%s\n%s\n%s\n%s\n\n",ROMTITLE,ROMVERSION,ROMDATE,ROMAUTHOR1,ROMAUTHOR2,ROMENV);
  
  {
    extern u32 __dtcm_lma;
    extern u32 __dtcm_start;
    extern u32 __dtcm_end;
    
    extern u32 __itcm_lma;
    extern u32 __itcm_start;
    extern u32 __itcm_end;
    
    _consolePrintf("ITCM0x%x,0x%x,0x%x\n",(u32)&__itcm_lma,(u32)&__itcm_start,(u32)&__itcm_end-(u32)&__itcm_start);
    _consolePrintf("DTCM0x%x,0x%x,0x%x\n",(u32)&__dtcm_lma,(u32)&__dtcm_start,(u32)&__dtcm_end-(u32)&__dtcm_start);
    _consolePrintf("\n");
  }
  
  _consolePrintf("Initialize FAT file system.\n");
  if(FAT_InitFiles()==false){;
    _consolePrintf("Fatal error! FAT_InitFiles()==false;\n");
    while(1);
  }
  SetARM9_REG_WaitCR();
  
  const bool prf=false;
  
  if(prf==true) PrfStart();
  _consolePrintf("Load font12.glu font12.glf.\n");
  LoadSystemFont();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) PrfStart();
  _consolePrintf("Load global.ini.\n");
  LoadGlobalINI();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) PrfStart();
  _consolePrintf("Load /ReinMoon/savename.txt.\n");
  LoadSaveName();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) PrfStart();
  _consolePrintf("Initialize misc.\n");
  FileList_Init();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) PrfStart();
  _consolePrintf("Load default icons.\n");
  Icon_LoadDefault();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) PrfStart();
  _consolePrintf("Load subbg/icon_gbacart/icon_dscard.bmp\n");
  InitDSCard();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) PrfStart();
  _consolePrintf("Initialize sound.\n");
  Sound_Init();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) PrfStart();
  _consolePrintf("Initialize window.\n");
  win_Init();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) PrfStart();
  _consolePrintf("Load win*.bmp\n");
  win_LoadBitmap();
  if(prf==true) PrfEnd(-1);
  
  if(prf==true) ShowLogHalt();
  
  _consolePrintf("Start interrupt.\n");
  InitInterrupts();
  
  IPC6->heartbeat=1;
  swiWaitForVBlank();
  _consolePrintf("Vsync passed.\n");
  
  {
    u32 UserLanguage=(u32)-1;
    u32 Timeout=0x10000;
    
    while(UserLanguage==(u32)-1){
      UserLanguage=IPC6->UserLanguage;
      Timeout--;
      if(Timeout==0){
        _consolePrintf("NDS farmware language read error. ARM7CPU stopped...?\n");
        while(1);
      }
    }
    _consolePrintf("NDS farmware language ID : %d\n",UserLanguage);
    
    switch(UserLanguage){
      case 1: SetLang(EL_ENG); break; // eng
      case 2: SetLang(EL_ENG); break; // fre
      case 4: SetLang(EL_ENG); break; // ita
      case 3: SetLang(EL_ENG); break; // deu
      case 5: SetLang(EL_ENG); break; // esp
      case 0: SetLang(EL_JPN); break; // jpn
      default: SetLang(EL_ENG); break;
    }
  }
  
  {
    u16 KEYS_CUR;
    
    KEYS_CUR=(~REG_KEYINPUT)&0x3ff;
    while(KEYS_CUR!=0){
      KEYS_CUR=(~REG_KEYINPUT)&0x3ff;
    }
  }
  
  {
    extern void mainloop(void);
    mainloop();
  }
  
  glDefaultClassFree();
  
  ShowLogHalt();
}

// ---------------------------------------

static TransferRegion6 RIPC;

extern u16 KEYS_CUR;
u16 KEYS_CUR=0;

extern void ScanIPC(void);

void ScanIPC(void)
{
  MemCopy32DMA3((void*)IPC6,&RIPC,sizeof(TransferRegion6));
  
  KEYS_CUR=0;
  
  u32 btns=~RIPC.buttons;
  
  KEYS_CUR|=(~REG_KEYINPUT)&0x3ff;
  if((btns&IPC_PEN_DOWN)!=0) KEYS_CUR|=KEY_TOUCH;
  if((btns&IPC_X)!=0) KEYS_CUR|=KEY_X;
  if((btns&IPC_Y)!=0) KEYS_CUR|=KEY_Y;
  
  if(KEYS_CUR==(KEY_L|KEY_R|KEY_SELECT|KEY_START)){
    u8 *pbmp;
    u32 bmpsize;
    
    pScreenMain->Flip(false);
    pbmp=pScreenMain->pCanvas->CreateBMPImage(&bmpsize);
    if(pbmp!=NULL){
      FAT_FILE *fp=FAT_fopen("/capture0.bmp","w");
      FAT_fwrite(pbmp,1,bmpsize,fp);
      FAT_fclose(fp);
      safefree(pbmp); pbmp=NULL;
    }
    
    pbmp=pScreenMainOverlay->pCanvas->CreateBMPImage(&bmpsize);
    if(pbmp!=NULL){
      FAT_FILE *fp=FAT_fopen("/capture1.bmp","w");
      FAT_fwrite(pbmp,1,bmpsize,fp);
      FAT_fclose(fp);
      safefree(pbmp); pbmp=NULL;
    }
    
    pbmp=pScreenSub->pCanvas->CreateBMPImage(&bmpsize);
    if(pbmp!=NULL){
      FAT_FILE *fp=FAT_fopen("/capture2.bmp","w");
      FAT_fwrite(pbmp,1,bmpsize,fp);
      FAT_fclose(fp);
      safefree(pbmp); pbmp=NULL;
    }
    
    _consolePrintf("Captured!! halt.\n");
    ShowLogHalt();
  }
}

// -----------------------------------------

static u32 KeyRepeatLastKey;
static bool KeyRepeatFlag;
static u32 KeyRepeatCount;

void KeyRepeat_Flash(void)
{
  KeyRepeatLastKey=0;
  KeyRepeatFlag=false;
}

u32 KeyRepeat_On(u32 NowKey)
{
  if(NowKey!=KeyRepeatLastKey) KeyRepeatFlag=false;
  KeyRepeatLastKey=NowKey;
  
  if(KeyRepeatFlag==false){
    KeyRepeatFlag=true;
    KeyRepeatCount=GlobalINI.KeyRepeat.DelayCount;
    }else{
    if(KeyRepeatCount==0){
      KeyRepeatCount=GlobalINI.KeyRepeat.RateCount;
      }else{
      NowKey=0;
    }
  }
  
  return(NowKey);
}

u32 KeyRepeat_Proc(u32 NowKey)
{
  if(NowKey==0){
    KeyRepeat_Flash();
    }else{
    NowKey=KeyRepeat_On(NowKey);
  }
  
  return(NowKey);
}

void KeyRepeat_Delay(u32 Multiple)
{
  KeyRepeatCount=GlobalINI.KeyRepeat.DelayCount*Multiple;
}

// --------------------------------------------

void WaitKeyClear(void)
{
  ScanIPC();
  while(KEYS_CUR!=0){
    swiWaitForVBlank();
    ScanIPC();
  }
}

void WaitKeyDown(void)
{
  ScanIPC();
  while(KEYS_CUR==0){
    swiWaitForVBlank();
    ScanIPC();
  }
}

void WaitKeyPress(void)
{
  WaitKeyDown();
  WaitKeyClear();
}

static void ProcessKeys(u16 loadkeys)
{
  if(loadkeys & (KEY_START | KEY_SELECT | KEY_UP | KEY_DOWN | KEY_LEFT | KEY_RIGHT | KEY_A | KEY_B | KEY_X | KEY_Y | KEY_L | KEY_R)==0) return;
  
  int vector=0;
  
  if((loadkeys & KEY_UP)!=0) vector=-1;
  if((loadkeys & KEY_DOWN)!=0) vector=1;
  if((loadkeys & KEY_LEFT)!=0) vector=-3;
  if((loadkeys & KEY_RIGHT)!=0) vector=3;
  
  if(vector!=0){
    Sound_Start(ESND_Cursor);
    FileList_MoveCursor(vector);
    MemSet16DMA3(RGB15(0,0,0)|BIT15,pScreenMain->pCanvas->GetVRAMBuf(),pScreenMain->pCanvas->GetHeight()*pScreenMain->pCanvas->GetWidth()*2);
    FileList_DrawCanvas(true);
    
    loadkeys&=~(KEY_UP | KEY_DOWN | KEY_LEFT | KEY_RIGHT);
  }
  
  if(loadkeys==KEY_X){
    Execute_ShowHelp();
    loadkeys&=~KEY_X;
  }
  
  if(loadkeys==KEY_A){
    FileList_Execute_Execute_A();
    loadkeys&=~KEY_A;
  }
  
  if(loadkeys==KEY_B){
    FileList_Execute_Execute_B();
    loadkeys&=~KEY_B;
  }
  
  if(loadkeys==(KEY_SELECT | KEY_START)){
    Execute_CreateNewFile();
    FileList_Refresh();
    FileList_DrawCanvas(true);
    loadkeys&=~(KEY_SELECT | KEY_START);
  }
  
}


static void mainloop_ProcVsync(u32 VsyncCount)
{
  if(KeyRepeatFlag==true){
    if(KeyRepeatCount<=VsyncCount){
      KeyRepeatCount=0;
      }else{
      KeyRepeatCount-=VsyncCount;
    }
    ScanIPC(); // I[gs[gVsyncKEYSĎȂB
    }else{
    ScanIPC();
  }
  
  u32 NowKey=KEYS_CUR;
  
  NowKey=KeyRepeat_Proc(NowKey);
  
  ProcessKeys(NowKey);
  
  if(NowKey!=0) return;
  
  PoolDSCard();
}

void ShowSplashWindow(void)
{
  win_Init();
  
  win_HideWindow();
  
  win_SetTitleMsg(RGB15(0,0,0),ROMTITLE " " ROMVERSION,ROMTITLE " " ROMVERSION);
  
  const char *e0=ROMDATE;
  const char *e1=ROMAUTHOR1;
  const char *e2=ROMAUTHOR2;
  const char *e3=ROMENV;
  const char *e4="";
  const char *e5="X button : show online help";
  
  const char *j0=ROMDATE;
  const char *j1=ROMAUTHOR1;
  const char *j2=ROMAUTHOR2;
  const char *j3=ROMENV;
  const char *j4="";
  const char *j5="X{^ : ICwvJ";
  
  win_SetMsg(0,e0,j0);
  win_SetMsg(1,e1,j1);
  win_SetMsg(2,e2,j2);
  win_SetMsg(3,e3,j3);
  win_SetMsg(4,e4,j4);
  win_SetMsg(5,e5,j5);
  
  win_SetRectNum(0,0,0,0);
  
  win_DrawWindow();
  
  Sound_Start(ESND_WinOpen);
  
  TBitmap *pbm=Bitmap_LoadFromFile("/ReinMoon/skin/warning.bmp",false);
  if(pbm==NULL) return;
  Bitmap_Draw(pbm,pScreenSub->pCanvas,0,0);
  Bitmap_Free(pbm); pbm=NULL;
  
  win_ShowWindow();
  
  WaitKeyClear();
  WaitKeyPress();
  
  Sound_Start(ESND_WinClose);
  win_HideWindow();
  win_Free();
}

void mainloop(void)
{
  WaitKeyClear();
  
  KeyRepeatLastKey=0;
  KeyRepeatFlag=false;
  KeyRepeatCount=0;
  
  FileList_Refresh();
  FileList_DrawCanvas(false);
  
  PrintFreeMem();
  
  if(GlobalINI.System.SplashScreen==true) ShowSplashWindow();
  
  {
    u16 *pb=pScreenSub->pCanvas->GetVRAMBuf();
    MemSet16DMA3(0,pb,ScreenHeight*ScreenWidth*2);
  }
  
  MemCopy16DMA3(pSubBGBM->pBM,pScreenSub->pCanvas->GetVRAMBuf(),192*256*2);
  FileList_DrawCanvas(true);
  
  while(1){
    while(VsyncPassedCount==0){
      swiWaitForIRQ();
    }
    
    u32 VsyncCount;
    
    REG_IME=0;
    VsyncCount=VsyncPassedCount;
    VsyncPassedCount=0;
    REG_IME=1;
    
    mainloop_ProcVsync(VsyncCount);
  }
}

void win_ProcModal_mainloop(void *pcbProcVsync,void *pcbProcKey)
{
  bool (*cb_ProcVsync)(u32 VsyncCount)=(bool(*)(u32))pcbProcVsync;
  bool (*cb_ProcKey)(u16 loadkeys)=(bool(*)(u16))pcbProcKey;
  
  if((cb_ProcVsync==NULL)&&(cb_ProcKey==NULL)){
    _consolePrintf("Fatal error!! win_ProcModal_mainloop no set CallBack function.\n");
    ShowLogHalt();
  }
  
  while(1){
    while(VsyncPassedCount==0){
      swiWaitForIRQ();
    }
    
    u32 VsyncCount;
    
    REG_IME=0;
    VsyncCount=VsyncPassedCount;
    VsyncPassedCount=0;
    REG_IME=1;
    
    if(cb_ProcVsync!=NULL){
      if(cb_ProcVsync(VsyncCount)==false) break;
    }
    
    if(KeyRepeatFlag==true){
      if(KeyRepeatCount<=VsyncCount){
        KeyRepeatCount=0;
        }else{
        KeyRepeatCount-=VsyncCount;
      }
      ScanIPC(); // I[gs[gVsyncKEYSĎȂB
      }else{
      ScanIPC();
    }
    
    u32 NowKey=KEYS_CUR;
    
    NowKey=KeyRepeat_Proc(NowKey);
    
    if(cb_ProcKey!=NULL){
      if(cb_ProcKey(NowKey)==false) break;
    }
  }
}

#include "main_exec_body.h"

// --------------

