
void Execute_ShowHelp(void)
{
  Dialog_OnlineHelp();
}

#define cfinfo_TopIndex (2)

enum Ecf {ECF_FileToDS=0,ECF_DSToFile=1,ECF_Cancel=2,ECF_Count=3};

typedef struct {
  Ecf ECF;
  bool reqdraw;
} Tcfinfo;

Tcfinfo cfinfo;

static bool Execute_CurrentFile_ProcVsync(u32 VsyncCount)
{
  if(cfinfo.reqdraw==false) return(true);
  cfinfo.reqdraw=false;
  
  for(u32 idx=0;idx<ECF_Count;idx++){
    win_SetMsgColor(cfinfo_TopIndex+idx,RGB15(16,16,16));
  }
  
  win_SetMsgColor(cfinfo_TopIndex+cfinfo.ECF,RGB15(0,0,0));
  
  for(u32 idx=0;idx<3;idx++){
    win_DrawMsg(cfinfo_TopIndex+idx);
  }
  
  CglCanvas *pCanvas=pScreenMainOverlay->pCanvas;
  TRect r=win_GetRect();
  
  {
    TRect cr;
    cr.x=0;
    cr.y=0;
    cr.w=r.w;
    cr.h=1*32;
    win_DrawClientBG(cr);
  }
  
  switch(cfinfo.ECF){
    case ECF_FileToDS: {
      Bitmap_Draw(pIconDSCard,pCanvas,r.x+(0*32),r.y);
      Bitmap_Draw(pIconVectorL,pCanvas,r.x+(1*32),r.y);
      Bitmap_Draw(pIconGBACart,pCanvas,r.x+(2*32),r.y);
    } break;
    case ECF_DSToFile: {
      Bitmap_Draw(pIconDSCard,pCanvas,r.x+(0*32),r.y);
      Bitmap_Draw(pIconVectorR,pCanvas,r.x+(1*32),r.y);
      Bitmap_Draw(pIconGBACart,pCanvas,r.x+(2*32),r.y);
    } break;
    case ECF_Cancel: {
      win_DrawMsg(0);
    } break;
    case ECF_Count: break;
  }
  
  return(true);
}

static bool Execute_CurrentFile_ProcKey(u16 loadkeys)
{
  if(loadkeys==KEY_A) return(false);
  if(loadkeys==KEY_B){
    cfinfo.ECF=ECF_Cancel;
    return(false);
  }
  
  Ecf ECF=cfinfo.ECF;
  
  if(loadkeys==KEY_UP){
    if(ECF==(Ecf)0){
      ECF=(Ecf)((int)ECF_Count-1);
      }else{
      ECF=(Ecf)((int)ECF-1);
    }
  }
  if(loadkeys==KEY_DOWN){
    if(ECF==(ECF_Count-1)){
      ECF=(Ecf)0;
      }else{
      ECF=(Ecf)((int)ECF+1);
    }
  }
  
  if(cfinfo.ECF!=ECF){
    cfinfo.ECF=ECF;
    cfinfo.reqdraw=true;
    Sound_Start(ESND_Cursor);
  }
  
  return(true);
}

void Execute_CurrentFile(TFile *pFile)
{
  win_Init();
  
  win_HideWindow();
  
  win_SetTitleMsg(RGB15(0,0,0),"execute menu","상j[");
  
  const char *e0="Selected file -> DS card";
  const char *e1="DS card -> Selected file";
  const char *e2="Cancel";
  
  const char *j0="t@CDSJ[hɕB";
  const char *j1="DSJ[ht@CɕۑB";
  const char *j2="LZ";
  
  win_SetMsgHeight(20);
  
  win_SetMsg(0,pFile->lfn,pFile->lfn);
  
  win_SetMsg(cfinfo_TopIndex+0,e0,j0);
  win_SetMsg(cfinfo_TopIndex+1,e1,j1);
  win_SetMsg(cfinfo_TopIndex+2,e2,j2);
  
  win_SetRectNum(0,0,256-2-8-8,0);
  
  cfinfo.ECF=ECF_Cancel;
  cfinfo.reqdraw=true;
  
  win_SetCallback_ProcVsync((void*)Execute_CurrentFile_ProcVsync);
  win_SetCallback_ProcKey((void*)Execute_CurrentFile_ProcKey);
  
  Execute_CurrentFile_ProcVsync(0);
  win_DrawWindow();
  Sound_Start(ESND_WinOpen);
  win_FadeInWindow();
  
  WaitKeyClear();
  
  win_ProcModal();
  
  if(cfinfo.ECF==ECF_Cancel) Sound_Start(ESND_WinClose);
  
  win_FadeOutWindow();
  win_Free();
  
  switch(cfinfo.ECF){
    case ECF_FileToDS: {
      void Execute_CurrentFile_FileToDS(TFile *pFile);
      Execute_CurrentFile_FileToDS(pFile);
    } break;
    case ECF_DSToFile: {
      void Execute_CurrentFile_DSToFile(TFile *pFile);
      Execute_CurrentFile_DSToFile(pFile);
    } break;
    case ECF_Cancel: break;
    case ECF_Count: break;
  }
}

typedef struct {
  int SaveType;
  u32 size;
  u32 sizediv;
  u8 *pbuf;
  FAT_FILE *fp;
} TSaveBuf;

TSaveBuf SaveBuf={0,0,0,NULL,NULL};

static void SaveBuf_Free(void)
{
  TSaveBuf *psb=&SaveBuf;
  
  psb->SaveType=0;
  psb->size=0;
  psb->sizediv=0;
  
  if(psb->pbuf!=NULL){
    safefree(psb->pbuf); psb->pbuf=NULL;
  }
  
  if(psb->fp!=NULL){
    FAT_fclose(psb->fp); psb->fp=NULL;
  }
}

static bool SaveBuf_Init(const char *curpath,const char *lfn,const char *openmodestr)
{
  TSaveBuf *psb=&SaveBuf;
  
  psb->SaveType=cardmeGetType();
  psb->size=cardmeSize(psb->SaveType);
  psb->pbuf=(u8*)safemalloc(psb->size);
  psb->fp=NULL;
  
  psb->sizediv=psb->size/128;
  psb->sizediv/=256;
  if(psb->sizediv==0) psb->sizediv=1;
  psb->sizediv*=256;
//  psb->sizediv=psb->size/32; // rein_r15݊
  
  if(psb->SaveType<=0){
    Dialog_ErrorDialog("DS card error (SaveType)","DSJ[hG[ (SaveType)");
    SaveBuf_Free();
    return(false);
  }
  
  if(psb->size==0){
    Dialog_ErrorDialog("DS card error (size)","DSJ[hG[ (size)");
    SaveBuf_Free();
    return(false);
  }
  
  if(psb->pbuf==NULL){
    Dialog_ErrorDialog("Memory overflow","s");
    SaveBuf_Free();
    return(false);
  }
  
  FAT_chdir(curpath);
  
  psb->fp=FAT_fopen(lfn,openmodestr);
  
  if(psb->fp==NULL){
    Dialog_ErrorDialog("file open error","t@CI[vG[");
    SaveBuf_Free();
    return(false);
  }
  
  return(true);
}

void Execute_CurrentFile_FileToDS(TFile *pFile)
{
  if(Dialog_StartDialog("write to DS card","DSJ[hɏ")==false) return;
  Sound_FreeMemory();
  FileList_Free();
  
  if(SaveBuf_Init(curpath,pFile->lfn,"r")==false) return;
  
  TSaveBuf *psb=&SaveBuf;
  
  Sound_StartLoop(ESND_Process);
  
  ProgressWindow_Open("read from file...","t@Cǂݍ݁c");
  for(u32 pos=0;pos<psb->size;pos+=psb->sizediv){
    ProgressWindow_SetPos(pos,psb->size);
    FAT_fread(&psb->pbuf[pos],1,psb->sizediv,psb->fp);
  }
  ProgressWindow_Close();
  
  if(psb->SaveType==3){
    ProgressWindow_Open("Erasing type 3 flash memory...","Type3tbVłc");
    int res=cardmeChipErase();
    ProgressWindow_Close();
    if(res<0){
      Sound_StopLoop();
      SaveBuf_Free();
      Dialog_ErrorDialog("Type3 failed eraserly.","DSJ[h̏Ɏs܂B");
      return;
    }
  }
  
  ProgressWindow_Open("write to DS card...","DSJ[hɏݒc");
  for(u32 pos=0;pos<psb->size;pos+=psb->sizediv){
    ProgressWindow_SetPos(pos,psb->size);
    cardmeWriteEeprom(pos,&psb->pbuf[pos],psb->sizediv,psb->SaveType);
  }
  ProgressWindow_Close();
  
  Sound_StopLoop();
  
  SaveBuf_Free();
  
  Dialog_CompliteDialog("Writing was completed to the DS card.","DSJ[hւ݂̏܂B");
}

void Execute_CurrentFile_DSToFile(TFile *pFile)
{
  if(Dialog_StartDialog("read from DS card","DSJ[hǂݍ")==false) return;
  Sound_FreeMemory();
  FileList_Free();
  
  Dialog_ErrorDialog("not implement function","̋@\\łB");
}

void Execute_CreateNewFile(void)
{
  if(Dialog_StartDialog("create new file","VKt@C쐬")==false) return;
  Sound_FreeMemory();
  FileList_Free();
  
  TGameData *pgd;
  
  {
    int SaveType=cardmeGetType();
    if(SaveType<=0){
      Dialog_ErrorDialog("not support save type.","Ή̃Z[u^CvłB");
      return;
    }
    
    u32 DetectSaveSize=GetDetectSaveSize();
    u8 *pSaveData=(u8*)safemalloc(DetectSaveSize);
    cardmeReadEeprom(0,pSaveData,DetectSaveSize,SaveType);
    pgd=GetGameDataFromSaveData((char*)pSaveData,DetectSaveSize);
    if(pgd==NULL) pgd=&GameDataDummy;
    safefree(pSaveData); pSaveData=NULL;
  }
  
  char lfn[1024];
  lfn[0]=(char)0;
  
  {
    TiniNewFilename *pNewFilename=&GlobalINI.NewFilename;
    
    if(pNewFilename->AddGameID==true){
      strcat(lfn,pgd->ID);
    }
    if(pNewFilename->AddENGName==true){
      char buf[256];
      
      strcpy(buf,pgd->ENGName);
      
      char *ptest=buf;
      
      while(*ptest!=(char)0){
        char c=*ptest;
        if(c<0x20) c=' ';
        if(('!'<=c)&&(c<='/')) c=' ';
        if((':'<=c)&&(c<='@')) c=' ';
        if(('['<=c)&&(c<='^')) c=' ';
        if('{'<=c) c=' ';
        *ptest=c;
        ptest++;
      }
      
      ptest--;
      
      while(*ptest==' '){
        *ptest=0;
        ptest++;
      }
      
      ptest=buf;
      
      while(*ptest!=(char)0){
        if((ptest[0]==' ')&&(ptest[1]==' ')){
          char *pc=ptest;
          while(*pc!=(char)0){
            pc[0]=pc[1];
            pc++;
          }
          }else{
          ptest++;
        }
      }
      
      if(pNewFilename->ENGNameLimit!=0) buf[pNewFilename->ENGNameLimit]=(char)0;
      
      strcat(lfn,"_");
      strcat(lfn,buf);
    }
    if((pNewFilename->AddDateYear==true)||(pNewFilename->AddDateMonth==true)||(pNewFilename->AddDateDay==true)){
      strcat(lfn,"_");
    }
    if(pNewFilename->AddDateYear==true){
      char buf[256];
      snprintf(buf,256,"%04d",2000+RIPC.rtc_year);
      strcat(lfn,buf);
    }
    if(pNewFilename->AddDateMonth==true){
      char buf[256];
      snprintf(buf,256,"%02d",RIPC.rtc_month);
      strcat(lfn,buf);
    }
    if(pNewFilename->AddDateDay==true){
      char buf[256];
      snprintf(buf,256,"%02d",RIPC.rtc_day);
      strcat(lfn,buf);
    }
    if((pNewFilename->AddTimeHour==true)||(pNewFilename->AddTimeMin==true)||(pNewFilename->AddTimeSec==true)){
      strcat(lfn,"_");
    }
    if(pNewFilename->AddTimeHour==true){
      char buf[256];
      u32 h=RIPC.rtc_hours;
      if(52<=h) h-=(52-12);
      snprintf(buf,256,"%02d",h);
      strcat(lfn,buf);
    }
    if(pNewFilename->AddTimeMin==true){
      char buf[256];
      snprintf(buf,256,"%02d",RIPC.rtc_minutes);
      strcat(lfn,buf);
    }
    if(pNewFilename->AddTimeSec==true){
      char buf[256];
      snprintf(buf,256,"%02d",RIPC.rtc_seconds);
      strcat(lfn,buf);
    }
  }
  
  strcat(lfn,".sav");
  
  _consolePrintf("save to %s\n",lfn);
  
  FAT_FreeFiles();
  FAT_InitFiles();
  
  if(SaveBuf_Init(curpath,lfn,"w")==false) return;
  
  TSaveBuf *psb=&SaveBuf;
  
  Sound_StartLoop(ESND_Process);
  
  ProgressWindow_Open("read from DS card...","DSJ[hǂݍ݁c");
  for(u32 pos=0;pos<psb->size;pos+=psb->sizediv){
    ProgressWindow_SetPos(pos,psb->size);
    cardmeReadEeprom(pos,&psb->pbuf[pos],psb->sizediv,psb->SaveType);
  }
  ProgressWindow_Close();
  
  ProgressWindow_Open("write to file...","t@Cɏݒc");
  for(u32 pos=0;pos<psb->size;pos+=psb->sizediv){
    ProgressWindow_SetPos(pos,psb->size);
    FAT_fwrite(&psb->pbuf[pos],1,psb->sizediv,psb->fp);
  }
  ProgressWindow_Close();
  
  Sound_StopLoop();
  
  SaveBuf_Free();
  
  FAT_FreeFiles();
  FAT_InitFiles();
  
  Dialog_CompliteDialog("Writing was completed to a file.","t@Cւ݂̏܂B");
}

