
static u16 *pIcon_Unknown;
static u16 *pIcon_Folder;
static u16 *pIcon_Back;

static void loadiconbmp(u16 *pIcon,char *fn)
{
  if(pIcon==NULL) return;
  
  TBitmap *pbm=Bitmap_LoadFromFile(fn,true);
  
  if(pbm==NULL){
    MemSet16DMA3(0,pIcon,32*32*2);
    _consolePrintf("icon load error %s\n",fn);
    return;
  }
  
  if((pbm->Width!=32)||(pbm->Height!=32)){
    MemSet16DMA3(0,pIcon,32*32*2);
    _consolePrintf("not icon size (32x32) %s\n",fn);
    }else{
    MemCopy16DMA3(pbm->pBM,pIcon,32*32*2);
  }
  
  Bitmap_Free(pbm); pbm=NULL;
}

static void Icon_LoadDefault(void)
{
  pIcon_Unknown=(u16*)safemalloc(32*32*2);
  loadiconbmp(pIcon_Unknown,"/ReinMoon/skin/icon_unknown.bmp");
  pIcon_Folder=(u16*)safemalloc(32*32*2);
  loadiconbmp(pIcon_Folder,"/ReinMoon/skin/icon_folder.bmp");
  pIcon_Back=(u16*)safemalloc(32*32*2);
  loadiconbmp(pIcon_Back,"/ReinMoon/skin/icon_back.bmp");
}

static bool LoadIconBM(const EFileType EFT,const char *GameID,u16 *pBM)
{
  switch(EFT){
    case EFT_None: return(false); break;
    case EFT_Back: {
      MemCopy16DMA3(pIcon_Back,pBM,32*32*2);
      return(true);
    } break;
    case EFT_Path: {
      MemCopy16DMA3(pIcon_Folder,pBM,32*32*2);
      return(true);
    } break;
    case EFT_File: {
      // next
    } break;
    default: return(false); break;
  }
  
  FAT_FILE *fp=NULL;
  
  char tmp[5];
  strcpy(tmp,GameID);
  tmp[4]=(char)0;
  
  if(strcmp(tmp,"----")!=0){
    if(fp==NULL){
      char fn[MAX_FILENAME_LENGTH];
      snprintf(fn,MAX_FILENAME_LENGTH,"/ReinMoon/banner/%s.ban",tmp);
      fp=FAT_fopen(fn,"r");
    }
    if(fp==NULL){
      char fn[MAX_FILENAME_LENGTH];
      tmp[3]='J';
      snprintf(fn,MAX_FILENAME_LENGTH,"/ReinMoon/banner/%s.ban",tmp);
      fp=FAT_fopen(fn,"r");
    }
    if(fp==NULL){
      char fn[MAX_FILENAME_LENGTH];
      tmp[3]='E';
      snprintf(fn,MAX_FILENAME_LENGTH,"/ReinMoon/banner/%s.ban",tmp);
      fp=FAT_fopen(fn,"r");
    }
  }
  
  if(fp!=NULL){
    u32 *bm=(u32*)safemalloc(32*32/2);
    u16 *pal=(u16*)safemalloc(0x10*2);
    
    FAT_fseek(fp,8+0x020,SEEK_SET);
    FAT_fread(bm,1,32*32/2,fp);
    FAT_fseek(fp,8+0x220,SEEK_SET);
    FAT_fread(pal,1,0x10*2,fp);
    
    pal[0]=0;
    for(u32 idx=1;idx<0x10;idx++){
      pal[idx]|=BIT15;
    }
    
    u32 *psrc=bm;
    
    for(u32 y=0;y<4;y++){
      for(u32 x=0;x<4;x++){
        u16 *pdst=&pBM[(y*8*32)+x*8];
        for(u32 yo=0;yo<8;yo++){
          u32 data=*psrc++;
          for(u32 xo=0;xo<8;xo++){
            pdst[xo]=pal[data&0x0f]; data>>=4;
          }
          pdst+=32;
        }
      }
    }
    
    safefree(bm); bm=NULL;
    safefree(pal); pal=NULL;
    
    FAT_fclose(fp);
    return(true);
  }
  
  MemCopy16DMA3(pIcon_Unknown,pBM,32*32*2);
  
  return(true);
}

static void DrawIcon(const u16 *psrcbm,u16 *pdstbm,const u32 dstwidth)
{
  for(u32 x=0;x<32;x++){
    if(*psrcbm!=0) *pdstbm=(((*psrcbm&0x7bde)>>1)+((*pdstbm&0x7bde)>>1))|BIT15;
    pdstbm++; psrcbm++;
  }
  pdstbm+=dstwidth-32;
  
  for(u32 y=1;y<31;y++){
    if(*psrcbm!=0) *pdstbm=(((*psrcbm&0x7bde)>>1)+((*pdstbm&0x7bde)>>1))|BIT15;
    pdstbm++; psrcbm++;
    
    for(u32 x=1;x<31;x++){
      if(*psrcbm!=0) *pdstbm=*psrcbm;
      pdstbm++; psrcbm++;
    }
    
    if(*psrcbm!=0) *pdstbm=(((*psrcbm&0x7bde)>>1)+((*pdstbm&0x7bde)>>1))|BIT15;
    pdstbm++; psrcbm++;
    
    pdstbm+=dstwidth-32;
  }
  
  for(u32 x=0;x<32;x++){
    if(*psrcbm!=0) *pdstbm=(((*psrcbm&0x7bde)>>1)+((*pdstbm&0x7bde)>>1))|BIT15;
    pdstbm++; psrcbm++;
  }
  pdstbm+=dstwidth-32;
}

