
#include <nds.h>
#include <string.h>
#include <stdio.h>

#include "savename.h"

#include "_const.h"
#include "_console.h"
#include "memtool.h"

static u32 GameDatasCount;
static TGameData *pGameDatas;

static u32 GameDatasFindCount;
static TGameData **ppGameDatasFind;

static u32 DetectSaveSize;

static char* GetReleaseNumber(TGameData *pGameData,char *pstr)
{
  u32 num=0;
  
  while(*pstr!=','){
    num*=10;
    num+=((u32)*pstr)-((u32)'0');
    pstr++;
  }
  
  pGameData->ReleaseNumber=num;
  
  return(pstr+1);
}

static char* GetID(TGameData *pGameData,char *pstr)
{
  u32 len=0;
  
  while(*pstr!=','){
    pGameData->ID[len]=*pstr;
    len++;
    pstr++;
  }
  pGameData->ID[len]=(char)0;
  
  return(pstr+1);
}

static char* GetENGName(TGameData *pGameData,char *pstr)
{
  u32 len=0;
  char s[1024];
  
  while(*pstr!=','){
    s[len]=*pstr;
    len++;
    pstr++;
  }
  s[len]=(char)0;
  
  pGameData->ENGName=(char*)safemalloc(len+1);
  memcpy(pGameData->ENGName,s,len+1);
  
  return(pstr+1);
}

static char* GetJPNName(TGameData *pGameData,char *pstr)
{
  if(*pstr==','){
    u32 len=strlen(pGameData->ENGName);
    pGameData->JPNName=(UnicodeChar*)safemalloc((len+1)*2);
    StrConvert_Ank2Unicode(pGameData->ENGName,pGameData->JPNName);
    
    }else{
    
    u32 len=0;
    UnicodeChar us[512];
    UnicodeChar *pus=&us[0];
    
    StrConvert_UTF82Unicode(pstr,us);
    
    while(*pus!=','){
      len++;
      pus++;
    }
    *pus=(UnicodeChar)0;
    
    pGameData->JPNName=(UnicodeChar*)safemalloc((len+1)*2);
    memcpy(pGameData->JPNName,us,(len+1)*2);
  }
  
  while(*pstr!=','){
    pstr++;
  }
  
  return(pstr+1);
}

static char* GetOffset(TGameData *pGameData,char *pstr)
{
  if((pstr[0]!='0')||(pstr[1]!='x')){
    _consolePrintf("ParseError:%s to hex",pstr);
    while(1);
  }
  
  pstr+=2;
  
  u32 ofs=0;
  
  while(*pstr!=','){
    ofs<<=4;
    char c=*pstr++;
    if(('0'<=c)&&(c<='9')) ofs|=0x00+(c-'0');
    if(('a'<=c)&&(c<='f')) ofs|=0x0a+(c-'a');
    if(('A'<=c)&&(c<='F')) ofs|=0x0a+(c-'A');
  }
  
  pGameData->Offset=ofs;
  
  return(pstr+1);
}

static char* GetData(TGameData *pGameData,char *pstr)
{
  u32 datalen=0;
  u8 data[1024];
  
  while(*pstr!=','){
    if(*pstr!='\\'){
      data[datalen]=(u8)*pstr;
      datalen++;
      pstr++;
      }else{
      if(pstr[1]!='x'){
        _consolePrintf("ParseError:%s to hex",pstr);
        while(1);
      }
      
      u8 b=0;
      char c;
      
      c=pstr[2];
      if(('0'<=c)&&(c<='9')) b|=0x00+(c-'0');
      if(('a'<=c)&&(c<='f')) b|=0x0a+(c-'a');
      if(('A'<=c)&&(c<='F')) b|=0x0a+(c-'A');
      b<<=4;
      c=pstr[3];
      if(('0'<=c)&&(c<='9')) b|=0x00+(c-'0');
      if(('a'<=c)&&(c<='f')) b|=0x0a+(c-'a');
      if(('A'<=c)&&(c<='F')) b|=0x0a+(c-'A');
      
      data[datalen]=b;
      datalen++;
      pstr+=4;
    }
  }
  data[datalen]=0;
  
  pGameData->DataSize=datalen;
  pGameData->Data=(u8*)safemalloc(datalen+1);
  memcpy(pGameData->Data,data,datalen+1);
  
  return(pstr+1);
}

void LoadGameDatas(char *UTF8Text,int size)
{
  const u32 SaveDatasMax=1024;
  
  GameDatasCount=0;
  pGameDatas=(TGameData*)safemalloc(SaveDatasMax*sizeof(TGameData));
  if(pGameDatas==NULL){
    _consolePrintf("Fatal error! pGameDatas=malloc(%d)==NULL;\n",SaveDatasMax*sizeof(TGameData));
    while(1);
  }
  
  while(1){
    if((UTF8Text[0]=='0')&&(UTF8Text[1]==',')) return;
    
    u32 linebufcount;
    char linebuf[1024];
    
    linebufcount=0;
    linebuf[linebufcount]=(char)0;
    
    while((*UTF8Text!=0x0d)&&(*UTF8Text!=0x0a)){
      linebuf[linebufcount]=*UTF8Text;
      linebufcount++;
      UTF8Text++;
    }
    linebuf[linebufcount]=(char)0;
    UTF8Text++;
    
    if((linebufcount!=0)&&(linebuf[0]!='#')){
      char *ps=&linebuf[0];
      TGameData *pGameData=&pGameDatas[GameDatasCount];
      GameDatasCount++;
      
      ps=GetReleaseNumber(pGameData,ps);
      ps=GetID(pGameData,ps);
      ps=GetENGName(pGameData,ps);
      ps=GetJPNName(pGameData,ps);
      ps=GetOffset(pGameData,ps);
      GetData(pGameData,ps);
    }
  }
}

void CreateGameDatasFind(void)
{
  GameDatasFindCount=0;
  for(u32 idx=0;idx<GameDatasCount;idx++){
    TGameData *pgd=&pGameDatas[idx];
    if(pgd->DataSize!=0) GameDatasFindCount++;
  }
  
  ppGameDatasFind=(TGameData**)safemalloc(GameDatasFindCount*4);
  if(ppGameDatasFind==NULL){
    _consolePrintf("Fatal error! pGameDatasFind=malloc(%d)==NULL;\n",GameDatasFindCount*sizeof(TGameData));
    while(1);
  }
  
  GameDatasFindCount=0;
  for(u32 idx=0;idx<GameDatasCount;idx++){
    TGameData *pgd=&pGameDatas[idx];
    if(pgd->DataSize!=0){
      ppGameDatasFind[GameDatasFindCount]=pgd;
      GameDatasFindCount++;
    }
  }
  
  for(u32 idx1=0;idx1<GameDatasFindCount-1;idx1++){
    for(u32 idx2=idx1+1;idx2<GameDatasFindCount;idx2++){
      TGameData *pgdf1=ppGameDatasFind[idx1];
      TGameData *pgdf2=ppGameDatasFind[idx2];
      if(pgdf1->DataSize<pgdf2->DataSize){
        ppGameDatasFind[idx1]=pgdf2;
        ppGameDatasFind[idx2]=pgdf1;
        }else{
        if((pgdf1->ID[3]!='J')&&(pgdf2->ID[3]=='J')){
          ppGameDatasFind[idx1]=pgdf2;
          ppGameDatasFind[idx2]=pgdf1;
        }
      }
    }
  }
  
  DetectSaveSize=0;
  
  for(u32 idx=0;idx<GameDatasFindCount;idx++){
    TGameData *pgdf=ppGameDatasFind[idx];
    u32 size=pgdf->Offset+pgdf->DataSize;
    if(DetectSaveSize<size) DetectSaveSize=size;
  }
  
  _consolePrintf("Enable save data items = %d\n",GameDatasFindCount);
  _consolePrintf("DetectSaveSize = %dbyte\n",DetectSaveSize);
}

TGameData *GetGameDataFromSaveData(char *SaveData,int SaveDataSize)
{
  for(u32 idx=0;idx<GameDatasFindCount;idx++){
    TGameData *pgdf=ppGameDatasFind[idx];
    
    if(pgdf->DataSize!=0){
      if(memcmp(&SaveData[pgdf->Offset],pgdf->Data,pgdf->DataSize)==0) return(pgdf);
    }
  }
  return(NULL);
}

u32 GetDetectSaveSize(void)
{
  u32 res=DetectSaveSize;
  
  res=(res+0xff) & (~0xff);
  
  if(8192<res) res=8192; // max 8kbyte.
  
  return(res);
}

