
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <NDS.h>
#include "../../ipc6.h"

#include "_console.h"
#include "_const.h"
#include "memtool.h"
#include "gba_nds_fat.h"

#include "inifile.h"

#include "sound.h"

#define Multiple (2)

typedef struct {
  char *fn;
  u32 Freq;
  u32 BufOffset;
  u32 BufCount;
  s16 *lbuf,*rbuf;
} TSound;

static TSound Sound[ESND_Stop+1];

typedef struct {
  u16 wFormatTag;
  u16 nChannels;
  u32 nSamplesPerSec;
  u32 nAvgBytesPerSec;
  u16 nBlockAlign;
  u16 wBitsPerSample;
  u16 cbSize;
} WAVEFORMATEX;

static void LoadWaveFile(TSound *pSnd,char *fn)
{
  pSnd->fn=fn;
  if(pSnd->fn==NULL) return;
  
  FAT_FILE *fp=FAT_fopen(pSnd->fn,"r");
  if(fp==NULL){
    _consolePrintf("File not found %s.\n",pSnd->fn);
    pSnd->fn=NULL;
    return;
  }
  
  {
    WAVEFORMATEX wfex;
    
    FAT_fseek(fp,0x14,SEEK_SET);
    FAT_fread(&wfex,sizeof(WAVEFORMATEX),1,fp);
    
    if(wfex.wFormatTag!=0x0001){
      _consolePrintf("Illigal CompressFormat Error. wFormatTag=0x%x\n",wfex.wFormatTag);
      FAT_fclose(fp);
      pSnd->fn=NULL;
      return;
    }
    
    if(wfex.nChannels!=2){
      _consolePrintf("Channels Error. nChannels=%d\n",wfex.nChannels);
      FAT_fclose(fp);
      pSnd->fn=NULL;
      return;
    }
    
    if(wfex.wBitsPerSample!=16){
      _consolePrintf("Bits/Sample Error. wBitsPerSample=%d\n",wfex.wBitsPerSample);
      FAT_fclose(fp);
      pSnd->fn=NULL;
      return;
    }
    
    pSnd->Freq=wfex.nSamplesPerSec;
  }
  
  FAT_fseek(fp,0,SEEK_SET);
  
  // find "data"
  {
    u32 ofs=0;
    
    char *buf=(char*)safemalloc(256);
    
    u32 size=FAT_fread(buf,1,256,fp);
    if(size<4){
      _consolePrintf("can not find data chunk.\n");
      ofs=0;
      }else{
      while(1){
        if(buf[ofs]=='d'){
          if((buf[ofs+1]=='a')&&(buf[ofs+2]=='t')&&(buf[ofs+3]=='a')) break;
        }
        ofs++;
        if(ofs==(size-4)){
          _consolePrintf("can not find data chunk.\n");
          ofs=0;
          return;
        }
      }
    }
    
    safefree(buf); buf=NULL;
    
    if(ofs==0){
      FAT_fclose(fp);
      pSnd->fn=NULL;
      return;
    }
    
    FAT_fseek(fp,ofs+4,SEEK_SET);
  }
  
  u32 BufCount;
  FAT_fread(&BufCount,4,1,fp);
  
  pSnd->BufOffset=FAT_ftell(fp);
  pSnd->BufCount=BufCount/2/2; // 16bit 2ch only.
  pSnd->lbuf=NULL;
  pSnd->rbuf=NULL;
  
  FAT_fclose(fp);
}

static bool WaveFile_LoadBody(TSound *pSnd)
{
  if(pSnd->fn==NULL) return(false);
  if((pSnd->lbuf!=NULL)&&(pSnd->rbuf!=NULL)) return(true);
  
//  _consolePrintf("%d,0x%x,0x%x\n",pSnd->Freq,pSnd->BufOffset,pSnd->BufCount);
  
  FAT_FILE *fp=FAT_fopen(pSnd->fn,"r");
  
  FAT_fseek(fp,pSnd->BufOffset,SEEK_SET);
  
  s16 *preadbuf=(s16*)safemalloc(pSnd->BufCount*2*2);
  if(preadbuf==NULL){
    _consolePrintf("can not allocate read buffer. (%dbyte)\n",pSnd->BufCount*2*2);
    FAT_fclose(fp);
    return(false);
  }
  FAT_fread(&preadbuf[0],2*2,pSnd->BufCount,fp);
  
  pSnd->lbuf=(s16*)safemalloc(pSnd->BufCount*Multiple*2);
  pSnd->rbuf=(s16*)safemalloc(pSnd->BufCount*Multiple*2);
  if((pSnd->lbuf==NULL)||(pSnd->rbuf==NULL)){
    _consolePrintf("can not allocate sound buffer. (%dbyte)\n",pSnd->BufCount*2*2);
    if(pSnd->lbuf!=NULL){
      safefree(pSnd->lbuf); pSnd->lbuf=NULL;
    }
    if(pSnd->rbuf!=NULL){
      safefree(pSnd->rbuf); pSnd->rbuf=NULL;
    }
    safefree(preadbuf); preadbuf=NULL;
    FAT_fclose(fp);
    return(false);
  }
  
  s32 lastl=0,lastr=0;
  
  for(u32 idx=0;idx<pSnd->BufCount;idx++){
    s32 psl=preadbuf[idx*2+0];
    s32 psr=preadbuf[idx*2+1];
    
    psl=(lastl+psl)/2;
    psr=(lastr+psr)/2;
    
    s16 *pdl=&pSnd->lbuf[idx*Multiple];
    s16 *pdr=&pSnd->rbuf[idx*Multiple];
    
    switch(Multiple){
      case 1: {
        pdl[0]=psl;
        pdr[0]=psr;
      } break;
      case 2: {
        pdl[0]=lastl;
        pdl[1]=(lastl+psl)/2;
        pdr[0]=lastr;
        pdr[1]=(lastr+psr)/2;
      } break;
      case 4: {
        s32 sample,half;
        
        sample=psl;
        half=(lastl+sample)/2;
        pdl[0]=lastl;
        pdl[1]=(lastl+half)/2;
        pdl[2]=half;
        pdl[3]=(half+sample)/2;
        
        sample=psr;
        half=(lastr+sample)/2;
        pdr[0]=lastr;
        pdr[1]=(lastl+half)/2;
        pdr[2]=half;
        pdr[3]=(half+sample)/2;
      } break;
    }
    lastl=psl;
    lastr=psr;
  }
  
  safefree(preadbuf); preadbuf=NULL;
  
  FAT_fclose(fp);
  
  return(true);
}

static bool WaveFile_FreeBody(TSound *pSnd)
{
  if(pSnd->fn==NULL) return(false);
  
  if(pSnd->lbuf!=NULL){
    safefree(pSnd->lbuf); pSnd->lbuf=NULL;
  }
  if(pSnd->rbuf!=NULL){
    safefree(pSnd->rbuf); pSnd->rbuf=NULL;
  }
  
  return(true);
}

void Sound_Init(void)
{
  CP15_DrainWriteBuffer();
  IPC6->IPCSND=EIPCSND_AllStop;
  while(IPC6->IPCSND!=EIPCSND_Null);
  
  LoadWaveFile(&Sound[ESND_Cursor],"/ReinMoon/se/cursor.wav");
  LoadWaveFile(&Sound[ESND_Execute],"/ReinMoon/se/execute.wav");
  LoadWaveFile(&Sound[ESND_Error],"/ReinMoon/se/error.wav");
  LoadWaveFile(&Sound[ESND_WinOpen],"/ReinMoon/se/winopen.wav");
  LoadWaveFile(&Sound[ESND_WinClose],"/ReinMoon/se/winclose.wav");
  LoadWaveFile(&Sound[ESND_Process],"/ReinMoon/se/process.wav");
  LoadWaveFile(&Sound[ESND_Complite],"/ReinMoon/se/complite.wav");
  LoadWaveFile(&Sound[ESND_Stop],NULL);
}

void Sound_FreeMemory(void)
{
  CP15_DrainWriteBuffer();
  IPC6->IPCSND=EIPCSND_AllStop;
  while(IPC6->IPCSND!=EIPCSND_Null);
  
  WaveFile_FreeBody(&Sound[ESND_Cursor]);
  WaveFile_FreeBody(&Sound[ESND_Execute]);
  WaveFile_FreeBody(&Sound[ESND_Error]);
  WaveFile_FreeBody(&Sound[ESND_WinOpen]);
  WaveFile_FreeBody(&Sound[ESND_WinClose]);
  WaveFile_FreeBody(&Sound[ESND_Process]);
  WaveFile_FreeBody(&Sound[ESND_Complite]);
  WaveFile_FreeBody(&Sound[ESND_Stop]);
}

void Sound_Start(ESound ESND)
{
  if(GlobalINI.System.EnableSE==false) return;
  
  TSound *pSnd=&Sound[ESND];
  
  if(WaveFile_LoadBody(pSnd)==true){
    IPC6->IPCSND_Freq=pSnd->Freq*Multiple;
    IPC6->IPCSND_BufCount=pSnd->BufCount*Multiple;
    IPC6->IPCSND_lbuf=pSnd->lbuf;
    IPC6->IPCSND_rbuf=pSnd->rbuf;
    
    CP15_DrainWriteBuffer();
    IPC6->IPCSND=EIPCSND_Play;
    while(IPC6->IPCSND!=EIPCSND_Null);
  }
}

void Sound_StartLoop(ESound ESND)
{
  if(GlobalINI.System.EnableSE==false) return;
  
  TSound *pSnd=&Sound[ESND];
  
  if(WaveFile_LoadBody(pSnd)==true){
    IPC6->IPCSND_Freq=pSnd->Freq*Multiple;
    IPC6->IPCSND_BufCount=pSnd->BufCount*Multiple;
    IPC6->IPCSND_lbuf=pSnd->lbuf;
    IPC6->IPCSND_rbuf=pSnd->rbuf;
    
    CP15_DrainWriteBuffer();
    IPC6->IPCSND=EIPCSND_PlayLoop;
    while(IPC6->IPCSND!=EIPCSND_Null);
  }
}

void Sound_StopLoop(void)
{
  CP15_DrainWriteBuffer();
  IPC6->IPCSND=EIPCSND_AllStop;
  while(IPC6->IPCSND!=EIPCSND_Null);
}

