
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <NDS.h>

#include "_console.h"
#include "_const.h"
#include "memtool.h"

#include "gba_nds_fat.h"

#include "unicode.h"
#include "glib/glib.h"
#include "lang.h"

#include "inifile.h"

#include "bitmap.h"

#include "window.h"

static UnicodeChar TitleMsg[256];
static u32 TitleMsgWidth;
static u16 TitleMsgColor;

#define msgmax (12)
static UnicodeChar msg[msgmax][256];
static u16 msgcol[msgmax];

static u32 MsgPadX,MsgPadY,MsgHeight;

static bool (*cb_ProcVsync)(u32 VsyncCount);
static bool (*cb_ProcKey)(u16 loadkeys);

static TRect WinRect;

static TBitmap *pTtlLBM,*pTtlCBM,*pTtlRBM;
static TBitmap *pFrmULBM,*pFrmURBM,*pFrmDLBM,*pFrmDRBM;
static TBitmap *pFrmLBM,*pFrmRBM,*pFrmUBM,*pFrmDBM;
static TBitmap *pBodyBM;

void win_LoadBitmap(void)
{
  pTtlLBM=Bitmap_LoadFromFile("/ReinMoon/skin/winttll.bmp",false);
  pTtlCBM=Bitmap_LoadFromFile("/ReinMoon/skin/winttlc.bmp",false);
  pTtlRBM=Bitmap_LoadFromFile("/ReinMoon/skin/winttlr.bmp",false);
  pFrmULBM=Bitmap_LoadFromFile("/ReinMoon/skin/winfrmul.bmp",false);
  pFrmURBM=Bitmap_LoadFromFile("/ReinMoon/skin/winfrmur.bmp",false);
  pFrmDLBM=Bitmap_LoadFromFile("/ReinMoon/skin/winfrmdl.bmp",false);
  pFrmDRBM=Bitmap_LoadFromFile("/ReinMoon/skin/winfrmdr.bmp",false);
  pFrmLBM=Bitmap_LoadFromFile("/ReinMoon/skin/winfrml.bmp",false);
  pFrmRBM=Bitmap_LoadFromFile("/ReinMoon/skin/winfrmr.bmp",false);
  pFrmUBM=Bitmap_LoadFromFile("/ReinMoon/skin/winfrmu.bmp",false);
  pFrmDBM=Bitmap_LoadFromFile("/ReinMoon/skin/winfrmd.bmp",false);
  pBodyBM=Bitmap_LoadFromFile("/ReinMoon/skin/winbody.bmp",false);
}

void win_Init(void)
{
  CglCanvas *pCanvas=pScreenMainOverlay->pCanvas;
  MemSet16DMA3(0,pCanvas->GetVRAMBuf(),pCanvas->GetHeight()*pCanvas->GetWidth()*2);
  
  TitleMsg[0]=(UnicodeChar)0;
  TitleMsgWidth=0;
  TitleMsgColor=0;
  
  MsgPadX=2;
  MsgPadY=2;
  MsgHeight=14;
  
  for(u32 idx=0;idx<msgmax;idx++){
    msg[idx][0]=(UnicodeChar)0;
    msgcol[idx]=RGB15(0,0,0)|BIT15;
  }
  
  cb_ProcVsync=NULL;
  cb_ProcKey=NULL;
  
  TRect *pwr=&WinRect;
  
  pwr->x=0;
  pwr->y=0;
  pwr->w=0;
  pwr->h=0;
}

void win_Free(void)
{
  win_Init();
}

void win_SetCallback_ProcVsync(void *pcb)
{
  cb_ProcVsync=(bool(*)(u32))pcb;
}

void win_SetCallback_ProcKey(void *pcb)
{
  cb_ProcKey=(bool(*)(u16))pcb;
}

void win_SetTitleMsg(const u16 col,const char *msge,const char *msgj)
{
  Lang_GetUniStr(msge,msgj,TitleMsg);
  TitleMsgWidth=pScreenMainOverlay->pCanvas->GetTextWidthW(TitleMsg)+1+1;
  TitleMsgColor=col|BIT15;
}

void win_SetMsgPadX(const u32 x)
{
  MsgPadX=x;
}

void win_SetMsgPadY(const u32 y)
{
  MsgPadX=y;
}

void win_SetMsgHeight(const u32 h)
{
  MsgHeight=h;
}

void win_SetMsg(const u32 idx,const char *msge,const char *msgj)
{
  Lang_GetUniStr(msge,msgj,msg[idx]);
}

void win_SetMsgColor(const u32 idx,const u16 col)
{
  msgcol[idx]=col|BIT15;
}

void win_SetRect(TRect r)
{
  TRect *pwr=&WinRect;
  
  *pwr=r;
  
  if(pwr->w==0){
    u32 maxwidth=0;
    for(u32 idx=0;idx<msgmax;idx++){
      if(msg[idx][0]!=(UnicodeChar)0){
        u32 width=pScreenMainOverlay->pCanvas->GetTextWidthW(msg[idx]);
        if(maxwidth<width) maxwidth=width;
      }
    }
    pwr->w=MsgPadX+maxwidth+MsgPadX;
  }
  
  if(pwr->h==0){
    u32 maxheight=0;
    for(u32 idx=0;idx<msgmax;idx++){
      if(msg[idx][0]!=(UnicodeChar)0){
        u32 height=(idx+1)*MsgHeight;
        if(maxheight<height) maxheight=height;
      }
    }
    pwr->h=MsgPadY+maxheight+MsgPadY;
  }
  
  if(pwr->x==0) pwr->x=(ScreenWidth-pwr->w)/2;
  if(pwr->y==0) pwr->y=(ScreenHeight-pwr->h)/2;
}

void win_SetRectNum(u32 x,u32 y,u32 w,u32 h)
{
  TRect r;
  
  r.x=x;
  r.y=y;
  r.w=w;
  r.h=h;
  
  win_SetRect(r);
}

TRect win_GetRect(void)
{
  return(WinRect);
}

u16* win_GetClientBuf(void)
{
  TRect *pwr=&WinRect;
  u16 *pb=pScreenMainOverlay->pCanvas->GetVRAMBuf();
  
  pb+=(pwr->y*ScreenWidth)+pwr->x;
  
  return(pb);
}

void win_DrawMsg(const u32 idx)
{
  TRect wr=WinRect;
  CglCanvas *pCanvas=pScreenMainOverlay->pCanvas;
  
  if(msg[idx][0]!=(UnicodeChar)0){
    pCanvas->SetFontTextColor(msgcol[idx]);
    pCanvas->TextOutW(wr.x+MsgPadX,wr.y+MsgPadY+(idx*MsgHeight)+((MsgHeight-12)/2),msg[idx]);
  }
}

void win_DrawClientBG(TRect r)
{
  TRect wr=WinRect;
  CglCanvas *pCanvas=pScreenMainOverlay->pCanvas;
  
  Bitmap_Draw2(pBodyBM,0,0,pCanvas,wr.x+r.x,wr.y+r.y,r.w,r.h);
}

void win_DrawWindow(void)
{
  TRect wr=WinRect;
  CglCanvas *pCanvas=pScreenMainOverlay->pCanvas;
  
  // Window Frame
  
  Bitmap_Draw2(pFrmULBM,0,0,pCanvas,wr.x-8,wr.y-8,8,8);
  Bitmap_Draw2(pFrmURBM,0,0,pCanvas,wr.x+wr.w,wr.y-8,8,8);
  Bitmap_Draw2(pFrmDLBM,0,0,pCanvas,wr.x-8,wr.y+wr.h,8,8);
  Bitmap_Draw2(pFrmDRBM,0,0,pCanvas,wr.x+wr.w,wr.y+wr.h,8,8);
  
  Bitmap_Draw2(pFrmLBM,0,0,pCanvas,wr.x-8,wr.y,8,wr.h);
  Bitmap_Draw2(pFrmRBM,0,0,pCanvas,wr.x+wr.w,wr.y,8,wr.h);
  Bitmap_Draw2(pFrmUBM,0,0,pCanvas,wr.x,wr.y-8,wr.w,8);
  Bitmap_Draw2(pFrmDBM,0,0,pCanvas,wr.x,wr.y+wr.h,wr.w,8);
  
  // Window Body
  
  Bitmap_Draw2(pBodyBM,0,0,pCanvas,wr.x,wr.y,wr.w,wr.h);
  
  // Window Title
  
  TRect wtr;
  
  wtr.x=wr.x-8;
  wtr.y=wr.y-8-pTtlLBM->Height;
  wtr.w=pTtlLBM->Width+TitleMsgWidth+pTtlRBM->Width;
  wtr.h=pTtlLBM->Height;
  
  Bitmap_Draw2(pTtlLBM,0,0,pCanvas,wtr.x,wtr.y,pTtlLBM->Width,wtr.h);
  Bitmap_Draw2(pTtlCBM,0,0,pCanvas,wtr.x+pTtlLBM->Width,wtr.y,TitleMsgWidth,wtr.h);
  Bitmap_Draw2(pTtlRBM,0,0,pCanvas,wtr.x+pTtlLBM->Width+TitleMsgWidth,wtr.y,pTtlRBM->Width,wtr.h);
  
  // Window Title Message
  
  pCanvas->SetFontTextColor(TitleMsgColor);
  pCanvas->TextOutW(wtr.x+pTtlLBM->Width+1,wtr.y+2,TitleMsg);
  
  // Client Message
  
  for(u32 idx=0;idx<msgmax;idx++){
    win_DrawMsg(idx);
  }
}

void win_ShowWindow(void)
{
  pScreenMainOverlay->SetAlpha(15);
}

void win_HideWindow(void)
{
  pScreenMainOverlay->SetAlpha(0);
}

extern void ScanIPC(void);
extern u16 KEYS_CUR;

void win_FadeInWindow(void)
{
  if(GlobalINI.System.EnableFadeWindow==false){
    pScreenMainOverlay->SetAlpha(15);
    return;
  }
  
  bool f=false;
  ScanIPC();
  if(KEYS_CUR!=0) f=true;
  
  for(u32 alpha=0;alpha<16;alpha++){
    pScreenMainOverlay->SetAlpha(alpha);
    swiWaitForVBlank();
    ScanIPC();
    if(f==true){
      if(KEYS_CUR==0) f=false;
      }else{
      if(KEYS_CUR!=0){
        pScreenMainOverlay->SetAlpha(15);
        return;
      }
    }
  }
}

void win_FadeOutWindow(void)
{
  if(GlobalINI.System.EnableFadeWindow==false){
    pScreenMainOverlay->SetAlpha(0);
    return;
  }
  
  bool f=false;
  ScanIPC();
  if(KEYS_CUR!=0) f=true;
  
  for(u32 alpha=0;alpha<16;alpha++){
    pScreenMainOverlay->SetAlpha(15-alpha);
    swiWaitForVBlank();
    ScanIPC();
    if(f==true){
      if(KEYS_CUR==0) f=false;
      }else{
      if(KEYS_CUR!=0){
        pScreenMainOverlay->SetAlpha(0);
        return;
      }
    }
  }
}

void win_ProcModal(void)
{
  void win_ProcModal_mainloop(void *pcbProcVsync,void *pcbProcKey);
  win_ProcModal_mainloop((void*)cb_ProcVsync,(void*)cb_ProcKey);
}

